--
-- TurnedOnAnimations
-- Specialization for animated parts via "i3d"-animations if the vehicle is turned on
--
-- @author  Stefan Maurus	
-- @date  07/11/14

-- Copyright © Stefan Maurus, www.stefanmaurus.de

--[[
	<turnedOnAnimations>
		<animation rootNode="" clip="" speed="" unloadingSpeed="">
			<sound index="" path="" volume="" unloadingVolume=""/>
		</animation>
		<animation rootNode="" clip="" speed="" unloadingSpeed=""/>		
	</turnedOnAnimations>
]]

TurnedOnAnimations = {};

function TurnedOnAnimations.prerequisitesPresent(specializations)
	return true;
end;

function TurnedOnAnimations:load(xmlFile)

	local i = 0;
	self.turnedOnAnimations = {};
	while true do
		local key = string.format("vehicle.turnedOnAnimations.animation(%d)", i);
		if not hasXMLProperty(xmlFile, key) then
			break;
		end;
		local rootNode = Utils.indexToObject(self.components, getXMLString(xmlFile, key.."#rootNode"));
		local charSet = getAnimCharacterSet(rootNode);
		local clip = getAnimClipIndex(charSet, getXMLString(xmlFile, key.."#clip"));
		
		local speed = Utils.getNoNil(getXMLFloat(xmlFile, key.."#speed"),1);
		local unloadingSpeed = Utils.getNoNil(getXMLFloat(xmlFile, key.."#unloadingSpeed"), 0);
		
		local turnOnOffFadeTime = Utils.getNoNil(getXMLFloat(xmlFile, key.."#turnOnOffFadeTime"),1)*1000;

		local currentFadeTime = 0;
		assignAnimTrackClip(charSet, 0, clip);
		setAnimTrackLoopState(charSet, 0, true);
		
		setAnimTrackSpeedScale(charSet, 0, speed);
		
		local soundKey = key..".sound";
		
		if hasXMLProperty(xmlFile, soundKey) then
			local node = Utils.indexToObject(self.components, getXMLString(xmlFile, soundKey.."#index"));
			local path = Utils.getFilename(getXMLString(xmlFile, soundKey.."#path"), self.baseDirectory);
			local volume = Utils.getNoNil(getXMLFloat(xmlFile, soundKey.."#volume"), 1);
			local unloadingVolume = Utils.getNoNil(getXMLFloat(xmlFile, soundKey.."#unloadingVolume"), volume);
			
			local radius = Utils.getNoNil(getXMLFloat(xmlFile, soundKey.."#radius"),15);
			local innerRadius = Utils.getNoNil(getXMLFloat(xmlFile, soundKey.."#innerRadius"),12);

			local sound = createAudioSource("sound(%d)", path, radius, innerRadius, volume, 0);
			link(node, sound);
			setVisibility(sound, false);
			table.insert(self.turnedOnAnimations, {charSet=charSet, speed=speed, turnOnOffFadeTime=turnOnOffFadeTime, currentFadeTime=currentFadeTime, unloadingSpeed=unloadingSpeed, soundNode=sound, unloadingVolume=unloadingVolume, volume=volume});
		else
			table.insert(self.turnedOnAnimations, {charSet=charSet, speed=speed, turnOnOffFadeTime=turnOnOffFadeTime, currentFadeTime=currentFadeTime, unloadingSpeed=unloadingSpeed});
		end;
		i = i + 1;
	end;
end;

function TurnedOnAnimations:delete()
	for _, anim in pairs(self.turnedOnAnimations) do
		if anim.soundNode ~= nil then
			setVisibility(anim.soundNode, false);
		end;
	end;
end;

function TurnedOnAnimations:readStream(streamId, connection)
end;

function TurnedOnAnimations:writeStream(streamId, connection) 
end;

function TurnedOnAnimations:mouseEvent(posX, posY, isDown, isUp, button)
end;

function TurnedOnAnimations:keyEvent(unicode, sym, modifier, isDown)
end;

function TurnedOnAnimations:update(dt)
	if self.isTurnedOn or self.tipState > 0 then	
		for _, anim in pairs(self.turnedOnAnimations) do
			if anim.currentFadeTime < anim.turnOnOffFadeTime then
				anim.currentFadeTime = math.min(anim.currentFadeTime+dt, anim.turnOnOffFadeTime);
			end;
		end;
	else
		for _, anim in pairs(self.turnedOnAnimations) do
			if anim.currentFadeTime > 0 then
				anim.currentFadeTime = math.max(anim.currentFadeTime-dt, 0);
			end;
		end;
	end;
end;

function TurnedOnAnimations:updateTick(dt)		
	if self.isTurnedOn or self.tipState > 0 then	
		if self.tipState > 0 then
			for _, anim in pairs(self.turnedOnAnimations) do
				enableAnimTrack(anim.charSet, 0);
				local f = anim.currentFadeTime/anim.turnOnOffFadeTime;
				if anim.turnOnOffFadeTime == 0 then
					f = 1;
				end;
				setAnimTrackSpeedScale(anim.charSet, 0, anim.unloadingSpeed*f);
				if anim.soundNode ~= nil then
					if anim.unloadingSpeed > 0 then
						setVisibility(anim.soundNode, true);
					end;
					if anim.volume ~= anim.unloadingVolume then
						local sample = getAudioSourceSample(anim.soundNode);
						setSampleVolume(sample, anim.unloadingVolume);
					end;
				end;
			end;
		else
			for _, anim in pairs(self.turnedOnAnimations) do
				if anim.soundNode ~= nil then
					setVisibility(anim.soundNode, true);
				end;
				enableAnimTrack(anim.charSet, 0);
				local f = anim.currentFadeTime/anim.turnOnOffFadeTime;
				if anim.turnOnOffFadeTime == 0 then
					f = 1;
				end;
				setAnimTrackSpeedScale(anim.charSet, 0, anim.speed*f);	
				if anim.soundNode ~= nil then
					setVisibility(anim.soundNode, true);
					if anim.volume ~= anim.unloadingVolume then
						local sample = getAudioSourceSample(anim.soundNode);
						setSampleVolume(sample, anim.volume);
					end;
				end;				
			end;
		end;
	else
		for _, anim in pairs(self.turnedOnAnimations) do
			if anim.currentFadeTime > 0 then
				local f = anim.currentFadeTime/anim.turnOnOffFadeTime;
				if anim.turnOnOffFadeTime == 0 then
					f = 1;
				end;
				setAnimTrackSpeedScale(anim.charSet, 0, anim.speed*f);		
			else
				disableAnimTrack(anim.charSet, 0);
			end;
			if anim.soundNode ~= nil then
				setVisibility(anim.soundNode, false);
			end;
		end;
	end;
end;

function TurnedOnAnimations:draw()
end;

function TurnedOnAnimations:onAttach()
end;

function TurnedOnAnimations:onDetach()	
end; 